###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.ip.base

class Algorithm:

    def __init__(self):
        self.radius = 1
        self.num_neighbors = 8

    def setup(self, parameters):
        self.radius = int(parameters.get('radius', self.radius))
        self.num_neighbors  = int(parameters.get('num-neighbors',  self.num_neighbors))

        return True


    def process(self, inputs, outputs):

        frames = inputs["input_frames"].data.value

        lbp = bob.ip.base.LBP(neighbors=self.num_neighbors, uniform=True, circular=True, radius=self.radius)
        feat_vector = numpy.zeros(lbp.max_label, dtype='float64')

        for i in range(frames.shape[0]):
          frame = frames[0]
          lbpimage = numpy.zeros(lbp.get_lbp_shape(frame), 'uint16') # allocating the image with lbp codes
          lbp(frame, lbpimage) # calculating the lbp image
          hist = bob.ip.base.histogram(lbpimage, 0, lbp.max_label-1, lbp.max_label)
          hist = hist / sum(hist)
          feat_vector  = feat_vector + hist

        feat_vector = feat_vector / frames.shape[0]

        outputs["lbp_feature_vector"].write({
            'value': feat_vector,
        })

        return True
