###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.ip.base
import bob.ip.color

class Algorithm:

    def __init__(self):
        self.normed_height = 64
        self.normed_width  = 64
        self.minimum_face_size = 50 # minimum size of a valid face


    def setup(self, parameters):
        self.normed_height = parameters.get('crop-height', self.normed_height)
        self.normed_width  = parameters.get('crop-width',  self.normed_width)
        self.minimum_face_size = parameters.get('minimum-face-size', self.minimum_face_size)

        return True


    def process(self, inputs, outputs):

        video = inputs["video"].data.value
        annotations = inputs["annotations"].data.value

        output = []

        for i in range(video.shape[0]): #range(video.shape[0], -1, -1):
          frame = bob.ip.color.rgb_to_gray(video[i,:,:,:])

          if annotations[i].height < self.minimum_face_size or annotations[i].width < self.minimum_face_size:
            continue # don't add the invalid frames

          cropped_face = bob.ip.base.crop(frame,
              int(getattr(annotations[i], 'top-left-y')),
              int(getattr(annotations[i], 'top-left-x')),
              int(annotations[i].height),
              int(annotations[i].width),
              )

          tempbbx = numpy.zeros((self.normed_height, self.normed_width), dtype='float64')
          bob.ip.base.scale(cropped_face, tempbbx) # normalization
          tempbbx += 0.5
          output.append(numpy.floor(tempbbx).astype('uint8'))

        outputs["cropped_faces"].write({'value': numpy.array(output)})

        return True
