###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import bob.ip.base
import bob.ip.color


class Algorithm:

    def __init__(self):
        self.crop_height = 80
        self.crop_width  = 64
        self.right_eye_y = 16
        self.right_eye_x = 15
        self.left_eye_y  = 16
        self.left_eye_x  = 48


    def setup(self, parameters):
        self.crop_height = parameters.get('crop-height', self.crop_height)
        self.crop_width  = parameters.get('crop-width',  self.crop_width)
        self.right_eye_y = parameters.get('right-eye-y', self.right_eye_y)
        self.right_eye_x = parameters.get('right-eye-x', self.right_eye_x)
        self.left_eye_y  = parameters.get('left-eye-y',  self.left_eye_y)
        self.left_eye_x  = parameters.get('left-eye-x',  self.left_eye_x)

        self.cropper = bob.ip.base.FaceEyesNorm(self.crop_height,
            self.crop_width, self.right_eye_y, self.right_eye_x,
            self.left_eye_y, self.left_eye_x)

        return True


    def process(self, inputs, outputs):
        image = bob.ip.color.rgb_to_gray(inputs["image"].data.value)
        eye_positions = inputs["eye_centers"].data

        outputs["cropped_image"].write({
            'value': self.cropper(
                image,
                float(eye_positions.right.y),
                float(eye_positions.right.x),
                float(eye_positions.left.y),
                float(eye_positions.left.x)
                )
        })

        return True
