###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.measure


class Algorithm:

    def __init__(self):
        self.positives_dev = []
        self.negatives_dev = []
        self.positives_test = []
        self.negatives_test = []

    def process(self, inputs, output):

        # accumulate the dev scores
        while inputs['scores_dev'].hasMoreData():
          inputs['scores_dev'].next()
          data_dev = inputs['scores_dev'].data
          self.positives_dev.extend([k.score for k in data_dev.scores if k.template_identity == data_dev.client_identity])
          self.negatives_dev.extend([k.score for k in data_dev.scores if k.template_identity != data_dev.client_identity])

        # accumulate the test scores
        data_test = inputs['scores_test'].data
        self.positives_test.extend([k.score for k in data_test.scores if k.template_identity == data_test.client_identity])
        self.negatives_test.extend([k.score for k in data_test.scores if k.template_identity != data_test.client_identity])

        # once all values are received, evaluate the scores
        if not(inputs.hasMoreData()):
            eer_threshold = bob.measure.eer_threshold(self.negatives_dev, self.positives_dev)
            far_dev, frr_dev   = bob.measure.farfrr(self.negatives_dev, self.positives_dev, eer_threshold)
            far_test, frr_test = bob.measure.farfrr(self.negatives_test, self.positives_test, eer_threshold)
            roc_points_dev     = bob.measure.roc(self.negatives_dev, self.positives_dev, 100)
            roc_points_test    = bob.measure.roc(self.negatives_test, self.positives_test, 100)

            # writes the output back to the platform
            output.write({
                'eer': numpy.float32((far_dev+frr_dev) / 2.),
                'hter': numpy.float32((far_test+frr_test) / 2.),
                'far_dev': numpy.float32(far_dev),
                'frr_dev': numpy.float32(frr_dev),
                'far_test': numpy.float32(far_test),
                'frr_test': numpy.float32(frr_test),
                'number_of_positives_dev': numpy.int32(len(self.positives_dev)),
                'number_of_negatives_dev': numpy.int32(len(self.negatives_dev)),
                'number_of_positives_test': numpy.int32(len(self.positives_test)),
                'number_of_negatives_test': numpy.int32(len(self.negatives_test)),
                'threshold': numpy.float32(eer_threshold),
                'roc_dev': {
                    "data":
                        [
                            {
                                "label": "development",
                                "x": roc_points_dev[0],
                                "y": roc_points_dev[1],
                            }
                        ]
                    },
                'roc_test': {
                    "data":
                        [
                            {
                                "label": "test",
                                "x": roc_points_test[0],
                                "y": roc_points_test[1],
                            }
                        ],
                    }
            })

        return True
