###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.core
import bob.learn.em


class Algorithm:

    def __init__(self):
        self.number_of_gaussians = 100
        self.max_iterations = 10
        self.data = []


    def setup(self, parameters):
        self.number_of_gaussians = parameters.get('number-of-gaussians',
                                                  self.number_of_gaussians)

        self.max_iterations = parameters.get('maximum-number-of-iterations',
                                             self.max_iterations)

        return True


    def process(self, inputs, outputs):
        self.data.append(inputs["features"].data.value)

        if not(inputs.hasMoreData()):
            # create array set used for training
            training_set = numpy.vstack(self.data)

            input_size = training_set.shape[1]

            # create the KMeans and UBM machine
            kmeans = bob.learn.em.KMeansMachine(int(self.number_of_gaussians), input_size)
            ubm = bob.learn.em.GMMMachine(int(self.number_of_gaussians), input_size)

            # create the KMeansTrainer
            kmeans_trainer = bob.learn.em.KMeansTrainer()
            kmeans_trainer.initialization_method = bob.learn.em.KMeansTrainer.RANDOM_NO_DUPLICATE
            kmeans_trainer.max_iterations = int(self.max_iterations)
            kmeans_trainer.convergence_threshold = 0.0001
            kmeans_trainer.rng = bob.core.random.mt19937(5489)

            # train using the KMeansTrainer
            kmeans_trainer.train(kmeans, training_set)

            (variances, weights) = kmeans.get_variances_and_weights_for_each_cluster(training_set)
            means = kmeans.means

            # initialize the GMM
            ubm.means = means
            ubm.variances = variances
            ubm.weights = weights

            # train the GMM
            trainer = bob.learn.em.ML_GMMTrainer(True, True, True)
            trainer.max_iterations = int(self.max_iterations)
            trainer.convergence_threshold = 0.0001
            trainer.rng = bob.core.random.mt19937(5489)
            trainer.train(ubm, training_set)

            # outputs data
            outputs["ubm"].write({
                'weights':              ubm.weights,
                'means':                ubm.means,
                'variances':            ubm.variances,
                'variance_thresholds':  ubm.variance_thresholds,
            })

        return True
