###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import bob.math

class Algorithm:

    def __init__(self):
        self.models = None

    def process(self, inputs, outputs):

        # retrieve all the templates once
        if self.models is None:
            self.models = {}
            group = inputs.groupOf('model')

            while group.hasMoreData():
                group.next()

                model_id = group['model_id'].data.value

                self.models[model_id] = {
                    'client_id' : group['model_client_id'].data.value,
                    'model' : group['model'].data.value,
                }


        # process the probe
        comparison_ids = inputs['comparison_ids'].data.value
        probe = inputs['probe'].data.value

        scores = []
        for comparison_id in comparison_ids:
            model_client_id = self.models[comparison_id]['client_id']

            score = - bob.math.chi_square(self.models[comparison_id]['model'],
                probe)

            scores.append({
                'template_identity': model_client_id,
                'score': score,
            })

        outputs['scores'].write(
          {
            'client_identity': inputs['probe_client_id'].data.value,
            'scores': scores
          },
        )

        return True
