###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import bob.core
import bob.learn.em


def gmm_from_data(data):
    """Unmangles a bob.learn.em.GMMMachine from a BEAT Data object"""

    dim_c, dim_d = data.means.shape
    gmm = bob.learn.em.GMMMachine(dim_c, dim_d)
    gmm.weights = data.weights
    gmm.means = data.means
    gmm.variances = data.variances
    gmm.variance_thresholds = data.variance_thresholds
    return gmm


def stats_from_data(data):
    """Unmangles a bob.learn.em.GMMStats from a BEAT Data object"""

    dim_c, dim_d = data.sum_px.shape
    stat = bob.learn.em.GMMStats(dim_c, dim_d)
    stat.t = long(data.t)
    stat.n = data.n
    stat.sum_px = data.sum_px
    stat.sum_pxx = data.sum_pxx
    return stat


class Algorithm:

    def __init__(self):
        self.isv_training_iterations = 10
        self.relevance_factor = 4.
        self.subspace_dimension_of_u = 50
        self.init_seed = 0
        self.data = {}
        self.ubm = None

    def setup(self, parameters):
        self.isv_training_iterations = parameters.get('isv-training-iterations', self.isv_training_iterations)
        self.relevance_factor = parameters.get('relevance-factor', self.relevance_factor)
        self.subspace_dimension_of_u = parameters.get('subspace-dimension-of-u', self.subspace_dimension_of_u)
        self.init_seed = parameters.get('init-seed', self.init_seed)
        return True


    def process(self, inputs, outputs):

        # retrieve the UBM once
        if self.ubm is None:
            self.ubm = gmm_from_data(inputs['ubm'].data)

        stats = stats_from_data(inputs["statistics"].data)
        c_id = inputs["client_id"].data.value
        if c_id in self.data.keys(): self.data[c_id].append(stats)
        else: self.data[c_id] = [stats]

        if not(inputs.hasMoreData()):
            # create array set used for training
            training_set = [v for k,v in self.data.iteritems()]

            isvbase = bob.learn.em.ISVBase(self.ubm, int(self.subspace_dimension_of_u))
            trainer = bob.learn.em.ISVTrainer(int(self.isv_training_iterations), self.relevance_factor)
            trainer.rng = bob.core.random.mt19937(int(self.init_seed))
            trainer.train(isvbase, training_set)

            # outputs data
            outputs["isvbase"].write({
                'subspace_u':              isvbase.u,
                'subspace_d':              isvbase.d,
            })

        return True
