###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import bob.learn.em


def gmm_from_data(data):
    """Unmangles a bob.learn.em.GMMMachine from a BEAT Data object"""

    dim_c, dim_d = data.means.shape
    gmm = bob.learn.em.GMMMachine(dim_c, dim_d)
    gmm.weights = data.weights
    gmm.means = data.means
    gmm.variances = data.variances
    gmm.variance_thresholds = data.variance_thresholds
    return gmm


def isvbase_from_data(data, ubm):
    """Unmangles a bob.learn.em.ISVBase from a BEAT Data object"""

    dim_cd, dim_u = data.subspace_u.shape
    isvbase = bob.learn.em.ISVBase(ubm, dim_u)
    isvbase.u = data.subspace_u
    isvbase.d = data.subspace_d
    return isvbase


def stats_from_data(data):
    """Unmangles a bob.learn.em.GMMStats from a BEAT Data object"""

    dim_c, dim_d = data.sum_px.shape
    stat = bob.learn.em.GMMStats(dim_c, dim_d)
    stat.t = long(data.t)
    stat.n = data.n
    stat.sum_px = data.sum_px
    stat.sum_pxx = data.sum_pxx
    return stat


class Algorithm:

    def __init__(self):
        self.ubm      = None
        self.isvbase  = None
        self.statistics = []
        self.isv_enroll_iterations = 1

    def setup(self, parameters):
        self.isv_enroll_iterations = parameters.get('isv-enroll-iterations', self.isv_enroll_iterations)
        return True

    def process(self, inputs, outputs):
        # retrieve the UBM once
        if self.ubm is None:
            inputs['ubm'].next()
            self.ubm = gmm_from_data(inputs['ubm'].data)

        # retrieve the ISVBase once
        if self.isvbase is None:
            inputs['isvbase'].next()
            self.isvbase = isvbase_from_data(inputs['isvbase'].data, self.ubm)

        # collect all the features for the current template
        self.statistics.append(stats_from_data(inputs["statistics"].data))


        # adapts the UBM GMM for the template (when all the features have been collected)
        if inputs["template_id"].isDataUnitDone():
            model = bob.learn.em.ISVMachine(self.isvbase)

            trainer = bob.learn.em.ISVTrainer()
            trainer.enrol(model, self.statistics, int(self.isv_enroll_iterations))

            # outputs data
            outputs["model"].write({
                'latent_z':             model.z,
            })

            self.statistics = []

        return True
