###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.learn.linear


def linear_machine_from_data(data):
    """Unmangles a bob.learn.linear.Machine from a BEAT Data object"""

    machine = bob.learn.linear.Machine(data.weights)
    machine.biases = data.biases
    machine.input_subtract = data.input_subtract
    machine.input_divide = data.input_divide
    return machine



class Algorithm:

    def __init__(self):
        self.pca_machine        = None
        self.lda_machine        = None
        self.projections        = []


    def process(self, inputs, outputs):

        # retrieve the linear machines once
        if self.pca_machine is None:
            inputs['subspace_pca'].next()
            self.pca_machine = linear_machine_from_data(inputs['subspace_pca'].data)
        if self.lda_machine is None:
            inputs['subspace_lda'].next()
            self.lda_machine = linear_machine_from_data(inputs['subspace_lda'].data)

        # collect all the image projections for the current template
        image = inputs['image'].data.value.flatten()
        projection = self.lda_machine.forward(self.pca_machine.forward(image))

        self.projections.append(projection)


        # generate the results (when all the images of the current template have been
        # projected)
        if inputs["id"].isDataUnitDone():
            outputs['projections'].write({
                'value': numpy.array(self.projections, dtype=numpy.float64)
            })

            self.projections = []

        return True
