###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import scipy.spatial


class Algorithm:

    def __init__(self):
        self.templates = None


    def process(self, inputs, outputs):

        # retrieve all the templates once
        if self.templates is None:

            self.templates = {}
            group = inputs.groupOf('template_projections')

            while group.hasMoreData():
                group.next()

                templates_id = group['template_id'].data.value

                self.templates[templates_id] = dict(
                    client_id = group['template_client_id'].data.value,
                    projections = group['template_projections'].data.value,
                )


        # process the probe
        comparison_ids  = inputs['comparison_ids'].data.value
        projections     = inputs['probe_projections'].data.value

        scores = []
        for comparison_id in comparison_ids:
            template_client_identity = self.templates[comparison_id]['client_id']

            distances = []

            # gets the min distance for all subprobes w.r.t to all subtemplates
            for subprobe in projections:
                distances.append(numpy.min([scipy.spatial.distance.euclidean(k, subprobe)
                                            for k in self.templates[comparison_id]['projections']]))

            # gets the min distance and inverts the scores so matching
            # numbers are numerically bigger than non-matching ones
            scores.append({
                'template_identity': self.templates[comparison_id]['client_id'],
                'score': -numpy.min(distances),
            })


        if inputs['probe_id'].isDataUnitDone():
            outputs['scores'].write({
                    'client_identity': inputs['probe_client_id'].data.value,
                    'scores': scores
                }
            )

        return True
