###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.learn.mlp


def mlp_from_data(data):
    """Unmangles a dict of bob.learn.mlp.Machine from a BEAT Data object"""

    n_inputs  = data.input_subtract.shape[0]
    n_outputs = data.biases[-1].value.shape[0]
    n_hidden  = len(data.biases) - 1
    shape = [n_inputs]
    for i in range(n_hidden): shape.append(data.biases[i].value.shape[0])
    shape.append(n_outputs)
    mlp = bob.learn.mlp.Machine(shape)

    mlp.input_subtract = data.input_subtract
    mlp.input_divide   = data.input_divide
    weights = []
    for v in data.weights: weights.append(v.value)
    biases = []
    for v in data.biases:  biases.append(v.value)
    mlp.weights = weights
    mlp.biases  = biases

    return mlp


class Algorithm:

    def __init__(self):
        self.model = None


    def process(self, inputs, outputs):

      # retrieve the model
      if self.model is None:
        inputs['model'].next()
        self.model = mlp_from_data(inputs['model'].data)

      # probe image
      image = inputs['image'].data.value.flatten()

      scores = {}
      scores = self.model(image)

      # Get the class_id with maximum score
      max_id = scores.argmax()
      outputs['scores'].write({
            'real_identity': inputs['class_id'].data.value,
            'estimated_identity': numpy.uint64(max_id),
            'score': scores[max_id]
         })

      return True
