###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.core
import bob.learn.mlp
import bob.learn.activation


class Algorithm:

    def __init__(self):
      self.data = []
      self.data_id = []
      self.number_of_hidden_units = 10
      self.number_of_iterations = 50
      self.seed = 0

    def setup(self, parameters):
      self.number_of_hidden_units = int(parameters.get('number-of-hidden-units',
                                                   self.number_of_hidden_units))
      self.number_of_iterations = int(parameters.get('number-of-iterations',
                                                   self.number_of_iterations))
      self.seed = int(parameters.get('seed', 0))
      return True

    def process(self, inputs, outputs):
      self.data.append(inputs["image"].data.value.flatten())
      y = -1*numpy.ones((1, 10), dtype=float)
      y[0, inputs["class_id"].data.value] = 1.
      self.data_id.append(y)

      if not(inputs.hasMoreData()):
        self.data = numpy.vstack(self.data)
        self.data_id = numpy.vstack(self.data_id)

         # Prepares MLP machine
        if self.number_of_hidden_units == 0:
          machine = bob.learn.mlp.Machine((28*28, 10))
        else:
          machine = bob.learn.mlp.Machine((28*28, self.number_of_hidden_units, 10))
        rng = bob.core.random.mt19937(self.seed)
        machine.randomize(rng)

        # Prepares MLP R-Prop trainer
        batch_size = self.data.shape[0]
        trainer = bob.learn.mlp.RProp(batch_size, bob.learn.mlp.SquareError(bob.learn.linear.HyperbolicTangent()))
        trainer.initialize(machine)
        # Launch training
        for it in range(self.number_of_iterations):
          trainer.train(machine, self.data, self.data_id)

        #import ipdb; ipdb.set_trace()
        weights = []
        for v in machine.weights: weights.append({'value': v})
        biases = []
        for v in machine.biases: biases.append({'value': v})
        outputs["model"].write({'input_subtract': machine.input_subtract,
                                'input_divide': machine.input_divide,
                                'weights': weights,
                                'biases': biases,
                                })

      return True
