###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.measure


class Algorithm:

    def __init__(self):
        self.positives = []
        self.negatives = []


    def process(self, inputs, output):

      # accumulate the scores
      data = inputs['scores'].data
      real_id = data.real_identity
      estimated_id = data.estimated_identity

      if real_id == estimated_id:
          self.positives.append(data.score)
      else:
          self.negatives.append(data.score)

      # once all values are received, evaluate the scores
      if not(inputs.hasMoreData()):
        cer = len(self.negatives) / float(len(self.negatives) + len(self.positives))
        roc_points    = bob.measure.roc(self.negatives, self.positives, 100)

        # writes the output back to the platform
        output.write({
                'cer': numpy.float32(cer),
                'number_of_positives': numpy.int32(len(self.positives)),
                'number_of_negatives': numpy.int32(len(self.negatives)),
                'roc': {
                    "data": {
                        [
                            "label": "roc",
                            "x": roc_points[0],
                            "y": roc_points[1],
                            ]
                        }
                    },
            })

      return True
