###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.learn.linear


class Algorithm:

    def __init__(self):
        self.number_of_pca_components = 5
        self.number_of_lda_components = 2
        self.data = {}


    def setup(self, parameters):
        self.number_of_pca_components = parameters.get('number-of-pca-components',
                                                   self.number_of_pca_components)
        self.number_of_lda_components = parameters.get('number-of-lda-components',
                                                   self.number_of_lda_components)
        return True

    def _project_data_for_lda(self, machine):
        tdata = []
        for client_id, client_files in self.data.iteritems():
            # at least two files per client are required!
            if len(client_files) < 2:
                # "Skipping client since the number of client files is only %d" %len(client_files)
                continue
            tdata.append(numpy.vstack([machine(feature) for feature in client_files]))
        return tdata

    def _perform_pca(self, pca_machine, training_set):
        """Perform PCA on data"""
        data = []
        for client_features in training_set:
            data.append(numpy.vstack([machine(feature) for feature in client_features]))
        return data

    def process(self, inputs, outputs):
        image = inputs["image"].data.value.flatten()
        c_id = inputs["client_id"].data.value
        if c_id in self.data.keys(): self.data[c_id].append(image)
        else: self.data[c_id] = [image]

        if not(inputs.hasMoreData()):
            # PCA
            data_pca = numpy.vstack([self.data[c_id] for c_id in self.data.keys()])
            trainer = bob.learn.linear.PCATrainer()
            pca_machine, eigen_values = trainer.train(data_pca)
            del data_pca # Reduce memory usage
            pca_machine.resize(pca_machine.shape[0], int(self.number_of_pca_components))
            # outputs data
            outputs["subspace_pca"].write({
                'input_subtract': pca_machine.input_subtract,
                'input_divide':   pca_machine.input_divide,
                'weights':        pca_machine.weights,
                'biases':         pca_machine.biases,
            })

            # LDA
            data_lda = self._project_data_for_lda(pca_machine)
            lda_trainer = bob.learn.linear.FisherLDATrainer()
            lda_machine, lda_variances = lda_trainer.train(data_lda)
            del data_lda # Reduce memory usage
            lda_machine.resize(lda_machine.shape[0], int(self.number_of_lda_components))
            # outputs data
            outputs["subspace_lda"].write({
                'input_subtract': lda_machine.input_subtract,
                'input_divide':   lda_machine.input_divide,
                'weights':        lda_machine.weights,
                'biases':         lda_machine.biases,
            })


        return True
