###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.measure


class Algorithm:

    def __init__(self):
        self.positives = []
        self.negatives = []
        self.n_bins = 20

    def process(self, inputs, output):

        # accumulate the scores
        data = inputs['scores'].data

        self.positives.extend([k.score for k in data.scores if k.template_identity == data.client_identity])
        self.negatives.extend([k.score for k in data.scores if k.template_identity != data.client_identity])



        # once all values are received, evaluate the scores
        if not(inputs.hasMoreData()):
            eer_threshold = bob.measure.eer_threshold(self.negatives, self.positives)
            far, frr      = bob.measure.farfrr(self.negatives, self.positives, eer_threshold)
            roc_points    = bob.measure.roc(self.negatives, self.positives, 100)

            histo_pos, bin_pos = numpy.histogram(self.positives, self.n_bins)
            histo_neg, bin_neg = numpy.histogram(self.negatives, self.n_bins)

            # writes the output back to the platform
            output.write({
                'eer': numpy.float32((far+frr) / 2.),
                'far': numpy.float32(far),
                'frr': numpy.float32(frr),
                'threshold': numpy.float32(eer_threshold),
                'number_of_positives': numpy.int32(len(self.positives)),
                'number_of_negatives': numpy.int32(len(self.negatives)),
                'scores_distribution': {
                    "data":
                        [
                            {
                                "label": "negative scores",
                                "x": bin_neg[:-1],
                                "y": histo_neg,
                            },
                            {
                                "label": "positive scores",
                                "x": bin_pos[:-1],
                                "y": histo_pos,
                            }
                        ]
                    },
                'roc': {
                    "data":
                        [
                            {
                                "label": "roc",
                                "false_positives": roc_points[0],
                                "false_negatives": roc_points[1],
                                "number_of_positives": numpy.uint64(len(self.positives)),
                                "number_of_negatives": numpy.uint64(len(self.negatives)),
                            }
                        ]
                    }
            })
        return True
