###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.measure

class Algorithm:

    def __init__(self):
        self.positives_dev = []
        self.negatives_dev = []
        self.positives_test = []
        self.negatives_test = []
        self.n_bins = 20


    def process(self, inputs, outputs):

        # accumulate the dev scores
        while inputs['scores_dev'].hasMoreData():
          inputs['scores_dev'].next()
          inputs['label_dev'].next()
          data_dev = inputs['scores_dev'].data.value
          label_dev = inputs['label_dev'].data.text
          if label_dev == 'real':
            self.positives_dev.append(data_dev)
          else:
            self.negatives_dev.append(data_dev)

        # accumulate the test scores
        data_test = inputs['scores_test'].data.value
        label_test = inputs['label_test'].data.text
        if label_test == 'real':
            self.positives_test.append(data_test)
        else:
            self.negatives_test.append(data_test)

        # once all values are received, evaluate the scores
        if not(inputs.hasMoreData()):
            eer_threshold      = bob.measure.eer_threshold(self.negatives_dev, self.positives_dev)
            far_dev, frr_dev   = bob.measure.farfrr(self.negatives_dev, self.positives_dev, eer_threshold)
            far_test, frr_test = bob.measure.farfrr(self.negatives_test, self.positives_test, eer_threshold)
            roc_points_dev     = bob.measure.roc(self.negatives_dev, self.positives_dev, 100)
            roc_points_test    = bob.measure.roc(self.negatives_test, self.positives_test, 100)

            dev_hist_pos, dev_bin_pos = numpy.histogram(self.positives_dev, self.n_bins)
            dev_hist_neg, dev_bin_neg = numpy.histogram(self.negatives_dev, self.n_bins)

            test_hist_pos, test_bin_pos = numpy.histogram(self.positives_test, self.n_bins)
            test_hist_neg, test_bin_neg = numpy.histogram(self.negatives_test, self.n_bins)

            eer_threshold = bob.measure.eer_threshold(self.negatives_dev, self.positives_dev)
            far_dev, frr_dev   = bob.measure.farfrr(self.negatives_dev, self.positives_dev, eer_threshold)
            far_test, frr_test = bob.measure.farfrr(self.negatives_test, self.positives_test, eer_threshold)
            roc_points_dev     = bob.measure.roc(self.negatives_dev, self.positives_dev, 100)
            roc_points_test    = bob.measure.roc(self.negatives_test, self.positives_test, 100)

            # writes the output back to the platform
            outputs.write({
                'dev_eer': numpy.float32((far_dev+frr_dev) / 2.),
                'test_hter': numpy.float32((far_test+frr_test) / 2.),
                'dev_far': numpy.float32(far_dev),
                'dev_frr': numpy.float32(frr_dev),
                'test_far': numpy.float32(far_test),
                'test_frr': numpy.float32(frr_test),
                'dev_positives': numpy.int32(len(self.positives_dev)),
                'dev_negatives': numpy.int32(len(self.negatives_dev)),
                'test_positives': numpy.int32(len(self.positives_test)),
                'test_negatives': numpy.int32(len(self.negatives_test)),
                'dev_eer_threshold': numpy.float32(eer_threshold),
                'dev_roc': {
                        "data":
                        [
                            {
                                "label": "roc_dev",
                                "false_positives": roc_points_dev[1],
                                "false_negatives": roc_points_dev[0],
                                "number_of_positives": numpy.uint64(len(self.positives_dev)),
                                "number_of_negatives": numpy.uint64(len(self.negatives_dev)),
                            }
                        ]
                    },
                'test_roc': {
                    "data":
                        [
                            {
                                "label": "roc_test",
                                "false_positives": roc_points_test[1],
                                "false_negatives": roc_points_test[0],
                                "number_of_positives": numpy.uint64(len(self.positives_test)),
                                "number_of_negatives": numpy.uint64(len(self.negatives_test)),
                            }
                        ]
                    },

                'dev_scores': {
                    "data":
                        [
                            {
                                "label": "attacks",
                                "x": dev_bin_neg[:-1],
                                "y": dev_hist_neg,
                            },
                            {
                                "label": "genuines",
                                "x": dev_bin_pos[:-1],
                                "y": dev_hist_pos,
                            }
                        ]
                    },
                'test_scores': {
                    "data":
                        [
                            {
                                "label": "attacks",
                                "x": test_bin_neg[:-1],
                                "y": test_hist_neg,
                            },
                            {
                                "label": "genuines",
                                "x": test_bin_pos[:-1],
                                "y": test_hist_pos,
                            }
                        ]
                    },
            })
        return True
