###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.ip.base

class Algorithm:

  def __init__(self):
    self.sigma0      = 1
    self.sigma1      = 2
    self.gamma       = 0.2
    self.kernel_size = 5
    self.threshold   = 10.0
    self.alpha       = 0.1

  def setup(self, parameters):
    self.sigma0      = parameters.get('sigma0', self.sigma0)
    self.sigma1      = parameters.get('sigma1', self.sigma1)
    self.gamma       = parameters.get('gamma', self.gamma)
    self.kernel_size = parameters.get('kernel_size', self.kernel_size)
    self.threshold   = parameters.get('threshold', self.threshold)
    self.alpha       = parameters.get('alpha', self.alpha)

    self.m_tantriggs = bob.ip.base.TanTriggs(gamma=float(self.gamma), sigma0=float(self.sigma0), sigma1=float(self.sigma1), radius=int(self.kernel_size), threshold=float(self.threshold), alpha=float(self.alpha))

    return True


  def process(self, inputs, outputs):

    image_gray      = inputs["image_gray"].data.value
    image_tantriggs = numpy.ndarray(image_gray.shape, numpy.float64)

    # perform Tan&Triggs normalization
    self.m_tantriggs(image_gray, image_tantriggs)

    outputs["tantriggs_image"].write({
            'value': image_tantriggs,
        })


    return True
