###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import bob
import xbob.db.casme2
import os
import numpy


class View:

    def setup(self, root_folder, outputs, parameters, force_start_index=None, force_end_index=None):
        self.root_folder = root_folder
        self.outputs     = outputs
        self.parameters  = parameters


        self.db          = xbob.db.casme2.Database()
        self.objs        = sorted(self.db.objects(protocol=parameters['protocol'], groups=parameters['group']), key=lambda x: x.id)
        self.next_index  = 0

        self.force_start_index = force_start_index
        self.force_end_index   = force_end_index

        # Retrieve only 'useful' data
        ### End index
        if self.force_end_index is not None:
            self.objs = self.objs[:self.force_end_index+1]
        ### Start index
        if self.force_start_index is not None:
            self.objs = self.objs[self.force_start_index:]
            self.next_index = self.force_start_index
        else:
            self.force_start_index = 0

        return True


    def done(self):
        return (self.next_index-self.force_start_index >= len(self.objs))


    def next(self):

        obj = self.objs[self.next_index-self.force_start_index]

        if self.outputs['file_id'].isConnected():
            self.outputs['file_id'].write({'value': numpy.uint64(obj.id)}, self.next_index)

        if self.outputs['client_id'].isConnected():
            self.outputs['client_id'].write({'value': numpy.uint64(obj.client_id)}, self.next_index)

        if self.outputs['image'].isConnected():

            frames          = obj.frames #geting the frames of a files

            f =  str(os.path.join(obj.make_path(self.root_folder),frames[0].filename))
            d    = bob.io.load(f)
            data = numpy.zeros(shape=(len(frames), d.shape[0], d.shape[1], d.shape[2]), dtype="uint8")
            data[0] = d
            for i in range(1,len(frames)):
                f =  str(os.path.join(obj.make_path(self.root_folder),frames[i].filename))
                d       = bob.io.load(f)
                data[i] = d

            data = {
                'value': data
            }

            self.outputs['image'].write(data, self.next_index)

        if self.outputs['emotion'].isConnected():
            self.outputs['emotion'].write({'text': obj.emotion[1:]}, self.next_index)

        self.next_index += 1

        return True

