###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import os
import bob.io.base
import bob.io.image
from bob.db.livdet2013 import Database


class All:

    def setup(self, root_folder, outputs, parameters, force_start_index=None, force_end_index=None):

        self.root_folder = root_folder
        self.outputs     = outputs

        self.db          = Database()

        # Retrieve database objects of interest
        self.objs = self.db.objects(
                groups=parameters['group'],
                classes=parameters.get('class'), #if None, get live/spoof
                protocols=parameters.get('protocol'), #the same as "Full"
                )

        # Filter according to start/end indexes to use
        self.start = force_start_index or 0 #from start
        force_end_index = force_end_index #None == all of it

        if force_end_index is not None: #platform uses inclusive ranges
            force_end_index += 1 #adjust to python ranges

        self.objs = self.objs[self.start:force_end_index]
        self.sent = 0 #objects already sent

        return True


    def done(self):
        return self.sent >= len(self.objs)


    def next(self):

        obj = self.objs[self.sent]
        index = self.sent + self.start

        if self.outputs['image'].isConnected():
            data = bob.io.base.load(obj.make_path(self.root_folder))
            self.outputs['image'].write({'value': data}, index)

        if self.outputs['spoof'].isConnected():
            self.outputs['spoof'].write({'value': not obj.is_live()}, index)

        self.sent += 1

        return True
