###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy
import bob.db.mnist


class View:

    def setup(self, root_folder, outputs, parameters, force_start_index=None, force_end_index=None):
        self.root_folder = root_folder
        self.outputs     = outputs

        db               = bob.db.mnist.Database(data_dir=self.root_folder)
        features, labels = db.data(groups=parameters['group'])
        self.features    = features
        self.labels      = labels
        self.next_index  = 0

        self.force_start_index = force_start_index
        self.force_end_index   = force_end_index

        # Retrieve only 'useful' data
        ### End index
        if self.force_end_index is None:
            self.force_end_index = self.features.shape[0]-1
        ### Start index
        if self.force_start_index is not None:
            self.next_index = self.force_start_index
        else:
            self.force_start_index = 0

        return True


    def done(self):
        return (self.next_index > self.force_end_index)


    def next(self):

        feat  = self.features[self.next_index,:]
        label = self.labels[self.next_index]

        if self.outputs['id'].isConnected():
            self.outputs['id'].write({'value': numpy.uint64(self.next_index)}, self.next_index)

        if self.outputs['class_id'].isConnected():
            self.outputs['class_id'].write({'value': numpy.uint64(label)}, self.next_index)

        if self.outputs['image'].isConnected():
            data = {
                'value': feat.reshape((28, 28))
            }

            self.outputs['image'].write(data, self.next_index)

        self.next_index += 1

        return True
