###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import bob
import numpy
import xbob.db.replay


from xbob.db.replay.driver import Interface
INFO = Interface()
SQLITE_FILE = INFO.files()[0]


class All:

    def setup(self, root_folder, outputs, parameters, force_start_index=None, force_end_index=None):
        self.root_folder = root_folder
        self.outputs     = outputs
        self.parameters  = parameters

        self.db               = xbob.db.replay.Database()

        if self.parameters['enroll']:
          self.objs_enroll    = sorted(self.db.objects(protocol=parameters['protocol'], groups=parameters['group'], cls='enroll'), key=lambda x: x.id)
        else:
          self.objs_enroll    = []
        self.objs_real        = sorted(self.db.objects(protocol=parameters['protocol'], groups=parameters['group'], cls='real'), key=lambda x: x.id)
        self.objs_att_fixed   = sorted(self.db.objects(protocol=parameters['protocol'], groups=parameters['group'], cls='attack', support='fixed'), key=lambda x: x.id)
        self.objs_att_hand    = sorted(self.db.objects(protocol=parameters['protocol'], groups=parameters['group'], cls='attack', support='hand'), key=lambda x: x.id)
        self.offset_enroll    = 0
        self.offset_real      = self.offset_enroll + len(self.objs_enroll)
        self.offset_att_fixed = self.offset_real + len(self.objs_real)
        self.offset_att_hand  = self.offset_att_fixed + len(self.objs_att_hand)
        self.next_index       = 0

        self.force_start_index = force_start_index
        self.force_end_index = force_end_index

        # Retrieve only 'useful' data
        ### Start index
        if self.force_start_index is not None:
            self.next_index = self.force_start_index
        ### End index
        if self.force_end_index is not None:
            if self.force_end_index < self.offset_real:
                self.objs_enroll = self.objs_enroll[:self.force_end_index+1]
                self.objs_real = []
                self.objs_att_fixed = []
                self.objs_att_hand = []
            elif self.force_end_index < self.offset_att_fixed:
                self.objs_real = self.objs_real[:self.force_end_index+1-self.offset_real]
                self.objs_att_fixed = []
                self.objs_att_hand = []
            elif self.force_end_index < self.offset_att_hand:
                self.objs_att_fixed = self.objs_att_fixed[:self.force_end_index+1-self.offset_att_fixed]
                self.objs_att_hand = []
            elif self.force_end_index < self.offset_att_hand + len(self.objs_att_hand) - 1:
                self.objs_att_hand = self.objs_att_hand[:self.force_end_index+1-self.offset_att_hand]

        return True


    def done(self):
        return (self.next_index >= len(self.objs_enroll) + len(self.objs_real) + len(self.objs_att_fixed) + len(self.objs_att_hand))


    def next(self):

        cls = ''
        attack_support = ''
        if self.next_index >= self.offset_att_hand:
          obj = self.objs_att_hand[self.next_index - self.offset_att_hand]
          cls = 'attack'
          attack_support = 'hand'
        elif self.next_index >= self.offset_att_fixed:
          obj = self.objs_att_fixed[self.next_index - self.offset_att_fixed]
          cls = 'attack'
          attack_support = 'fixed'
        elif self.next_index >= self.offset_real:
          obj = self.objs_real[self.next_index - self.offset_real]
          cls = 'real'
        else:
          obj = self.objs_enroll[self.next_index - self.offset_enroll]
          cls = 'enroll'

        if self.outputs['file_id'].isConnected():
            self.outputs['file_id'].write({'value': numpy.uint64(obj.id)}, self.next_index)

        if self.outputs['client_id'].isConnected():
            self.outputs['client_id'].write({'value': numpy.uint64(obj.client_id)}, self.next_index)

        if self.outputs['video'].isConnected():
            data = {
                'value': bob.io.load(obj.videofile(self.root_folder))
            }

            self.outputs['video'].write(data, self.next_index)

        if self.outputs['annotations'].isConnected():
            for i in range(50):
              annotations = obj.bbx(self.root_folder)

            annotations_list = []
            for i in range(annotations.shape[0]):
              annotations_list.append({'frame_id': numpy.uint64(annotations[i][0]),
                  'top-left-x': numpy.int32(annotations[i][1]), 'top-left-y':numpy.int32(annotations[i][2]),
                  'width':  numpy.int32(annotations[i][3]), 'height':  numpy.int32(annotations[i][4])
                })
            self.outputs['annotations'].write({'value': annotations_list}, self.next_index)


        if self.outputs['class'].isConnected():
            self.outputs['class'].write({'text': cls}, self.next_index)

        if self.outputs['attack_support'].isConnected():
            self.outputs['attack_support'].write({'text': attack_support}, self.next_index)
        self.next_index += 1

        return True


class Templates:
    # Reasoning: Each client may have a number of models in certain databases.
    # So, each model receives an unique identifier. Those identifiers are linked
    # to the client identifier and contain a number of images to generated the
    # model from.

    def setup(self, root_folder, outputs, parameters, force_start_index=None, force_end_index=None):
        self.root_folder            = root_folder
        self.outputs                = outputs
        self.parameters             = parameters

        self.db                     = xbob.db.replay.Database()

        self.template_ids           = sorted([c.id for c in self.db.clients() if c.set == parameters['group']])
        self.my_objs                = None
        self.objs                   = sorted(self.db.objects(protocol=parameters['protocol'],
                                                        groups=parameters['group'], cls='enroll'), key=lambda x: x.id)

        self.current_template_index = 0
        self.current_obj_index      = 0
        self.next_index             = 0

        self.force_start_index = force_start_index
        self.force_end_index   = force_end_index

        # Retrieve only 'useful' data
        ### End index
        #if self.force_end_index is not None:
        #    self.objs = self.objs[:self.force_end_index+1]
        ### Start index
        if self.force_start_index is None:
            self.force_start_index = 0
        while self.next_index < self.force_start_index:
            self.next()

        return True


    def done(self):
        return (self.current_template_index >= len(self.template_ids)) or (self.force_end_index is not None and self.force_end_index < self.next_index)


    def next(self):
        if self.my_objs is None:
            # probe for the specific objects concerning a given client
            self.my_objs = sorted([o for o in self.objs if o.client_id == self.template_ids[self.current_template_index]], key=lambda x: x.id)

            if self.force_start_index <= self.next_index and (self.force_end_index is None or self.force_end_index >= self.next_index):
                # For this database, 'self.template_ids[self.current_template_index]' corresponds to 'obj.client_id'
                if self.outputs['template_id'].isConnected():
                    self.outputs['template_id'].write({'value': numpy.uint64(self.template_ids[self.current_template_index])}, self.next_index+len(self.my_objs)-1)

                if self.outputs['client_id'].isConnected():
                    self.outputs['client_id'].write({'value': numpy.uint64(self.my_objs[0].client_id)}, self.next_index+len(self.my_objs)-1)

        obj = self.objs[self.current_obj_index]

        if self.outputs['file_id'].isConnected() or self.outputs['video'].isConnected() or self.outputs['annotations'].isConnected():
            if self.force_start_index <= self.next_index and (self.force_end_index is None or self.force_end_index >= self.next_index):
                if self.outputs['file_id'].isConnected():
                    self.outputs['file_id'].write({'value': numpy.uint64(obj.id)}, self.next_index)

                if self.outputs['video'].isConnected():
                    data = {
                        'value': bob.io.load(obj.videofile(self.root_folder))
                    }

                    self.outputs['video'].write(data, self.next_index)

                if self.outputs['annotations'].isConnected():
                    annotations = obj.bbx(self.root_folder)
                    annotations_list = []
                    for i in range(annotations.shape[0]):
                      annotations_list.append({'frame_id': numpy.uint64(annotations[i][0]),
                          'top-left-x': numpy.int32(annotations[i][1]), 'top-left-y':numpy.int32(annotations[i][2]),
                          'width':  numpy.int32(annotations[i][3]), 'height':  numpy.int32(annotations[i][4])
                        })

                    self.outputs['annotations'].write({'value': annotations_list}, self.next_index)

            self.next_index += 1
            self.current_obj_index += 1
        else:
            self.next_index += len(self.my_objs)
            self.current_obj_index = len(self.my_objs)

        if self.current_obj_index == len(self.my_objs):
            self.my_objs = None
            self.current_obj_index = 0
            self.current_template_index += 1

        return True


class ProbesReal:
    # Reasoning: Each client may have a number of probes. Each probe may be
    # composed of any number of images. So, each probe receives an unique
    # identifier. Those identifiers are linked to the client identifier and
    # contain a number of images to generated the match/comparison score. Each
    # probe must be matched against a number of models defined by a list of
    # client identifiers.

    def setup(self, root_folder, outputs, parameters, force_start_index=None, force_end_index=None):
        self.root_folder = root_folder
        self.outputs     = outputs
        self.parameters  = parameters

        self.db                     = xbob.db.replay.Database()

        self.template_ids           = sorted([c.id for c in self.db.clients() if c.set == parameters['group']])
        self.objs                   = sorted(self.db.objects(protocol=parameters['protocol'],
                                                      groups=parameters['group'], cls=parameters['class']), key=lambda x: x.id)

        self.next_index  = 0

        self.force_start_index = force_start_index
        self.force_end_index = force_end_index

        # Retrieve only 'useful' data
        ### Start index
        if self.force_start_index is not None:
            self.next_index = self.force_start_index
        ### End index
        if self.force_end_index is not None:
            if self.force_end_index < len(self.objs):
                self.objs = self.objs[:self.force_end_index+1]

        return True


    def done(self):
        return (self.next_index >= len(self.objs))


    def next(self):

        obj = self.objs[self.next_index]

        if self.outputs['file_id'].isConnected():
            self.outputs['file_id'].write({'value': numpy.uint64(obj.id)}, self.next_index)

        if self.outputs['probe_id'].isConnected():
            self.outputs['probe_id'].write({'value': numpy.uint64(obj.id)}, self.next_index)

        if self.outputs['client_id'].isConnected():
            self.outputs['client_id'].write({'value': numpy.uint64(obj.client_id)}, self.next_index)

        if self.outputs['template_ids'].isConnected():
            self.outputs['template_ids'].write({'value': numpy.uint64(self.template_ids)}, self.next_index)

        if self.outputs['video'].isConnected():
            data = {
                'value': bob.io.load(obj.videofile(self.root_folder))
            }

            self.outputs['video'].write(data, self.next_index)

        if self.outputs['annotations'].isConnected():
            annotations = obj.bbx(self.root_folder)
            annotations_list = []
            for i in range(annotations.shape[0]):
              annotations_list.append({'frame_id': numpy.uint64(annotations[i][0]),
                  'top-left-x': numpy.int32(annotations[i][1]), 'top-left-y':numpy.int32(annotations[i][2]),
                  'width':  numpy.int32(annotations[i][3]), 'height':  numpy.int32(annotations[i][4])
                })

            self.outputs['annotations'].write({'value': annotations_list}, self.next_index)

        self.next_index += 1

        return True


class ProbesAttack:
    # Reasoning: Each client may have a number of probes. Each probe may be
    # composed of any number of images. So, each probe receives an unique
    # identifier. Those identifiers are linked to the client identifier and
    # contain a number of images to generated the match/comparison score. Each
    # probe must be matched against a number of models defined by a list of
    # client identifiers.

    def setup(self, root_folder, outputs, parameters, force_start_index=None, force_end_index=None):
        self.root_folder = root_folder
        self.outputs     = outputs
        self.parameters  = parameters

        self.db                     = xbob.db.replay.Database()

        self.template_ids           = sorted([c.id for c in self.db.clients() if c.set == parameters['group']])
        self.objs_fixed             = sorted(self.db.objects(protocol=parameters['protocol'],
                                                      groups=parameters['group'], cls=parameters['class'], support='fixed'), key=lambda x: x.id)
        self.objs_hand              = sorted(self.db.objects(protocol=parameters['protocol'],
                                                      groups=parameters['group'], cls=parameters['class'], support='hand'), key=lambda x: x.id)
        self.offset_fixed = 0
        self.offset_hand  = self.offset_fixed + len(self.objs_fixed)

        self.next_index  = 0

        self.force_start_index = force_start_index
        self.force_end_index = force_end_index

        # Retrieve only 'useful' data
        ### Start index
        if self.force_start_index is not None:
            self.next_index = self.force_start_index
        ### End index
        if self.force_end_index is not None:
            if self.force_end_index < self.offset_hand:
                self.objs_fixed = self.objs_fixed[:self.force_end_index+1]
                self.objs_hand = []
            elif self.force_end_index < self.offset_hand + len(self.objs_hand) - 1:
                self.objs_hand = self.objs_hand[:self.force_end_index+1-self.offset_hand]

        return True


    def done(self):
        return (self.next_index >= len(self.objs_fixed) + len(self.objs_hand))


    def next(self):

        if self.next_index >= self.offset_hand:
          obj = self.objs_hand[self.next_index - self.offset_hand]
          attack_support = 'hand'
        else:
          obj = self.objs_fixed[self.next_index - self.offset_fixed]
          attack_support = 'fixed'

        if self.outputs['file_id'].isConnected():
            self.outputs['file_id'].write({'value': numpy.uint64(obj.id)}, self.next_index)

        if self.outputs['probe_id'].isConnected():
            self.outputs['probe_id'].write({'value': numpy.uint64(obj.id)}, self.next_index)

        if self.outputs['client_id'].isConnected():
            self.outputs['client_id'].write({'value': numpy.uint64(obj.client_id)}, self.next_index)

        if self.outputs['template_ids'].isConnected():
            self.outputs['template_ids'].write({'value': numpy.uint64([obj.client_id])}, self.next_index)

        if self.outputs['video'].isConnected():
            data = {
                'value': bob.io.load(obj.videofile(self.root_folder))
            }

            self.outputs['video'].write(data, self.next_index)

        if self.outputs['annotations'].isConnected():
            annotations = obj.bbx(self.root_folder)
            annotations_list = []
            for i in range(annotations.shape[0]):
              annotations_list.append({'frame_id': numpy.uint64(annotations[i][0]),
                  'top-left-x': numpy.int32(annotations[i][1]), 'top-left-y':numpy.int32(annotations[i][2]),
                  'width':  numpy.int32(annotations[i][3]), 'height':  numpy.int32(annotations[i][4])
                })

            self.outputs['annotations'].write({'value': annotations_list}, self.next_index)

        if self.outputs['attack_support'].isConnected():
            self.outputs['attack_support'].write({'text': attack_support}, self.next_index)

        self.next_index += 1

        return True
