###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

# Makes sure we won't require an X11 connection
import matplotlib
matplotlib.use('Agg')

import numpy
import itertools


class Plotter(baselib.Plotter):

    def setup(self, parameters):

        super(Plotter, self).setup(parameters)
        self.line_attributes = parameters.get('line_attributes', '').split('&')
        self.line_attributes = [k for k in self.line_attributes if k]
        return True


    def process(self, inputs):

        fig, ax = super(Plotter, self).prepare_canvas()

        args = []
        label = []

        single_experiment = len(inputs) == 1

        for xp_label, xp_data in inputs:
            for scatter in xp_data.data:

                # the data
                args.append(scatter.x * self.xaxis_multiplier)
                args.append(scatter.y * self.yaxis_multiplier)

                # the label
                if self.label: #gets the label from user overwritten input
                    label.append(self.label[len(label)%len(self.label)])
                else: #make-up label
                    if single_experiment:
                        label.append(scatter.label)
                    else:
                        text = [k for k in (xp_label, scatter.label) if k]
                        label.append('-'.join(text))

                # the line attributes
                if self.line_attributes:
                    args.append(self.line_attributes[len(label)%len(self.line_attributes)])

        lines = ax.plot(*args)
        if len(lines) > 1:
            ax.legend(lines, label, loc='best', fancybox=True, framealpha=0.5)

        super(Plotter, self).apply_parameters(ax)
        return super(Plotter, self).encode_figure(fig)
