###############################################################################
#                                                                             #
# Copyright (c) 2016 Idiap Research Institute, http://www.idiap.ch/           #
# Contact: beat.support@idiap.ch                                              #
#                                                                             #
# This file is part of the beat.examples module of the BEAT platform.         #
#                                                                             #
# Commercial License Usage                                                    #
# Licensees holding valid commercial BEAT licenses may use this file in       #
# accordance with the terms contained in a written agreement between you      #
# and Idiap. For further information contact tto@idiap.ch                     #
#                                                                             #
# Alternatively, this file may be used under the terms of the GNU Affero      #
# Public License version 3 as published by the Free Software and appearing    #
# in the file LICENSE.AGPL included in the packaging of this file.            #
# The BEAT platform is distributed in the hope that it will be useful, but    #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  #
# or FITNESS FOR A PARTICULAR PURPOSE.                                        #
#                                                                             #
# You should have received a copy of the GNU Affero Public License along      #
# with the BEAT platform. If not, see http://www.gnu.org/licenses/.           #
#                                                                             #
###############################################################################

import numpy


class Algorithm:

    def process(self, inputs, output):

        numpy.random.seed(0) #deterministic at every plot

        n_points = 100

        # data for the scatter plot, randomly generated
        x = numpy.arange(n_points, dtype=float)
        y = (n_points - x) + numpy.random.normal(0, 0.5, (n_points,))

        roc_data = {
            "data": [
                {
                    "label": "roc",
                    "number_of_positives": numpy.uint64(200),
                    "number_of_negatives": numpy.uint64(2000),
                    "false_positives": x/100.0,
                    "false_negatives": 1-(x/100.0),
                    },
                ]
            }

        scatter_data = {
            "data": [
                {
                    "label": "scatter",
                    "x": x/10,
                    "y": y,
                    },
                ]
            }

        bar_data = {
            "data": [
                {
                    "label": "bar",
                    "x": range(n_points),
                    "y": range(n_points),
                    },
                ]
            }

        output.write({
            'out_data': inputs['in_data'].data.value,
            'roc': roc_data,
            'plot': scatter_data,
            'bar_plot': bar_data,
        })

        return True
